import {
  Alert,
  Button,
  Card,
  Space,
  Flex,
  Row,
  Col,
  Typography,
  Grid,
  Skeleton,
  notification,
  Popconfirm,
  Tag,
} from "antd";
import {
  AppStyle,
  FormatDatetime,
  myFetch,
  showUnkownErrorNotification,
} from "../../utils";
import { Trans, useTranslation } from "react-i18next";
import { CheckOutlined, CloseOutlined } from "@ant-design/icons";
import { useEffect, useState } from "react";
import { ReactComponent as RocketLaunch } from "./rocket_launch_FILL1_wght400_GRAD0_opsz24.svg";
import { ReactComponent as Check } from "./task_alt_FILL0_wght400_GRAD0_opsz24.svg";
import CountUp from "react-countup";
import { getDaysLeft } from "../../Contexts/SideBarContext";

const { useBreakpoint } = Grid;

export default function PaymentPlan() {
  const { t } = useTranslation();

  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  const screenBreakpoint = useBreakpoint();

  const [isRequesting, setIsRequesting] = useState(true);
  const [requestData, setRequestData] = useState({
    payment_plan: 0,
    payment_plan_interval: 0,
    prices: [],
    payment_plan_trial_end: undefined,
    payment_plan_canceled_at: undefined,
  });
  const [requestingCheckout, setRequestingCheckout] = useState(false);

  const [selectedBillingPeriod, setSelectedBillingPeriod] = useState(0);

  const fetchPaymentPlan = () => {
    myFetch({
      url: "/payment",
      method: "GET",
      notificationApi,
      t,
    })
      .then((data) => {
        setRequestData(data);
        setIsRequesting(false);

        if (data.payment_plan_interval !== null) {
          setSelectedBillingPeriod(data.payment_plan_interval);
        }
      })
      .catch(() => {
        setIsRequesting(false);
        showUnkownErrorNotification(notificationApi, t);
      });
  };

  useEffect(() => fetchPaymentPlan(), []);

  const showAlertTrialEnds = () => {
    if (
      !isRequesting &&
      requestData.payment_plan === 0 &&
      requestData.payment_plan_trial_end &&
      !requestData.payment_plan_canceled_at
    ) {
      const daysLeft = getDaysLeft(requestData.payment_plan_trial_end);

      if (daysLeft <= 0) {
        return (
          <Alert
            message={
              <Typography.Text>
                {t("paymentPlan.alertTrialEnded")}
              </Typography.Text>
            }
            type="warning"
            showIcon
            closable
            style={{ marginBottom: 12 }}
          />
        );
      }

      return (
        <Alert
          message={
            <Typography.Text>
              {t("paymentPlan.alertDemoEndsIn.demoEndsIn")}{" "}
              <b>
                {daysLeft}{" "}
                {daysLeft === 1
                  ? t("common.unit.day")
                  : t("common.unit.daysLong")}
              </b>
              .{" "}
              {t("paymentPlan.alertDemoEndsIn.selectPlanToAccessFullFeatures")}
            </Typography.Text>
          }
          type="warning"
          closable
          showIcon
          style={{ marginBottom: 12 }}
        />
      );
    }
  };

  const showAlertPlanCancelled = () => {
    if (!isRequesting && requestData.payment_plan_canceled_at) {
      return (
        <Alert
          message={
            <Typography.Text>
              {t("paymentPlan.alertPlanCancelled", {
                date: FormatDatetime(requestData.payment_plan_canceled_at),
              })}
            </Typography.Text>
          }
          type="warning"
          showIcon
          closable
          style={{ marginBottom: 12 }}
        />
      );
    }
  };

  const percentageDiscount =
    requestData.prices.length > 0
      ? Math.round(
          100 -
            (100 / (requestData.prices[0].unit_amount * 12)) *
              requestData.prices[1].unit_amount
        )
      : 0;

  const handleButtonBuyPlan = () => {
    setRequestingCheckout(true);

    myFetch({
      method: "POST",
      url: "/payment/checkout",
      body: {
        lookupKey: `za-basic-${
          selectedBillingPeriod === 0 ? "monthly" : "yearly"
        }`,
      },
      notificationApi,
      t,
    })
      .then((data) => {
        if (data.url) {
          window.location.href = data.url;
          return;
        }

        if (data.status === "ok") {
          fetchPaymentPlan();
          setRequestingCheckout(false);
        }
      })
      .catch((error) => {
        console.log(error);
        setRequestingCheckout(false);
      });
  };

  const ButtonBuyPlan = ({ onClick }) => {
    return (
      <Button
        loading={requestingCheckout}
        shape="round"
        style={{ fontWeight: "bold" }}
        onClick={onClick}
      >
        {t("paymentPlan.buttonBuyPlan")}
      </Button>
    );
  };

  return (
    <>
      {notificationContextHolder}

      {showAlertTrialEnds()}
      {showAlertPlanCancelled()}

      <Card title={t("paymentPlan.title")}>
        <Row gutter={[16, 16]}>
          <Col xs={24} xl={12}>
            <Flex vertical>
              {t("paymentPlan.features", { returnObjects: true }).map(
                (feature, index) => (
                  <Space key={index}>
                    <Check />
                    <Typography.Text style={{ fontSize: 20 }}>
                      <Trans
                        i18nKey={feature.text}
                        values={{ bold: feature.bold }}
                        components={{ b: <b /> }}
                      />
                    </Typography.Text>
                  </Space>
                )
              )}
            </Flex>
          </Col>

          <Col xs={24} xl={12}>
            <Flex justify="center" style={{ marginBottom: 12 }}>
              <Space>
                {isRequesting ? (
                  <Skeleton.Button shape="round" size="default" active />
                ) : (
                  <Button
                    type={selectedBillingPeriod === 0 ? "primary" : "default"}
                    shape="round"
                    icon={
                      requestData.payment_plan_interval === 0 &&
                      (requestData.payment_plan_canceled_at === undefined ? (
                        <CheckOutlined />
                      ) : (
                        <CloseOutlined style={{ color: "#d10205" }} />
                      ))
                    }
                    onClick={() => setSelectedBillingPeriod(0)}
                  >
                    {t("paymentPlan.buttonMonthly")}
                  </Button>
                )}

                {isRequesting ? (
                  <Skeleton.Button shape="round" size="default" active />
                ) : (
                  <Button
                    type={selectedBillingPeriod === 1 ? "primary" : "default"}
                    shape="round"
                    icon={
                      requestData.payment_plan_interval === 1 &&
                      (requestData.payment_plan_canceled_at === undefined ? (
                        <CheckOutlined />
                      ) : (
                        <CloseOutlined style={{ color: "#d10205" }} />
                      ))
                    }
                    onClick={() => setSelectedBillingPeriod(1)}
                  >
                    <Space>
                      <span>{t("paymentPlan.buttonYearly")}</span>

                      {requestData.payment_plan_interval !== 1 && (
                        <Tag color="green">
                          -
                          <CountUp
                            start={
                              percentageDiscount > 10
                                ? percentageDiscount - 10
                                : 0
                            }
                            end={percentageDiscount}
                          />{" "}
                          %
                        </Tag>
                      )}
                    </Space>
                  </Button>
                )}
              </Space>
            </Flex>

            <Card style={{ backgroundColor: AppStyle.colors.primary }}>
              <Flex
                justify="space-between"
                vertical={screenBreakpoint.xs}
                align="center"
              >
                <div>
                  <RocketLaunch
                    style={{
                      width: 62,
                      height: 62,
                      backgroundColor: "#fff",
                      padding: 12,
                      borderRadius: 12,
                    }}
                  />
                </div>

                <Flex
                  vertical
                  style={{ marginBottom: screenBreakpoint.xs && 12 }}
                >
                  <Typography.Title
                    level={screenBreakpoint.xs ? 3 : 2}
                    style={{ color: "#fff" }}
                  >
                    {t("paymentPlan.businessPlan")}
                  </Typography.Title>

                  {isRequesting ? (
                    <Skeleton.Button shape="round" block active />
                  ) : (
                    <>
                      {selectedBillingPeriod ===
                      requestData.payment_plan_interval ? (
                        <Button
                          loading={requestingCheckout}
                          shape="round"
                          style={{ fontWeight: "bold" }}
                          onClick={() => {
                            setRequestingCheckout(true);

                            myFetch({
                              method: "GET",
                              url: "/payment/portal",
                            })
                              .then((data) => {
                                if (data.url) {
                                  window.location.href = data.url;
                                }
                              })
                              .catch((error) => {
                                console.log(error);
                                setRequestingCheckout(false);
                              });
                            return;
                          }}
                        >
                          {t("paymentPlan.buttonManagePlan")}
                        </Button>
                      ) : requestData.payment_plan === 0 ? (
                        <ButtonBuyPlan onClick={() => handleButtonBuyPlan()} />
                      ) : (
                        <Popconfirm
                          title={t("paymentPlan.popconfirmChangePlan.title")}
                          description={t(
                            "paymentPlan.popconfirmChangePlan.description"
                          )}
                          okText={t("common.button.confirm")}
                          cancelText={t("common.button.cancel")}
                          onConfirm={() => handleButtonBuyPlan()}
                        >
                          <ButtonBuyPlan />
                        </Popconfirm>
                      )}
                    </>
                  )}
                </Flex>

                <Flex vertical align="center">
                  {isRequesting ? (
                    <Skeleton.Button shape="round" size="large" active />
                  ) : (
                    <Typography.Title
                      level={screenBreakpoint.xs ? 3 : 2}
                      style={{
                        color: "#fff",
                        padding: 0,
                        margin: 0,
                        width: 76,
                        textAlign: "center",
                      }}
                    >
                      {`${
                        requestData.prices.find(
                          (price) =>
                            price.lookup_key ===
                            `za-basic-${
                              selectedBillingPeriod === 0 ? "monthly" : "yearly"
                            }`
                        ).unit_amount / (selectedBillingPeriod === 0 ? 1 : 12)
                      } €`}
                    </Typography.Title>
                  )}
                  <Typography.Text
                    style={{ color: "#fff", textAlign: "center" }}
                  >
                    /{t("paymentPlan.monthly")}
                  </Typography.Text>
                </Flex>
              </Flex>
            </Card>

            {selectedBillingPeriod === requestData.payment_plan_interval &&
              requestData.payment_plan_canceled_at !== undefined && (
                <Flex justify="center">
                  <Typography.Text type="secondary">
                    {t("paymentPlan.cancelledOn", {
                      date: FormatDatetime(
                        requestData.payment_plan_canceled_at
                      ),
                    })}
                  </Typography.Text>
                </Flex>
              )}

            {selectedBillingPeriod === 1 &&
              requestData.payment_plan_interval !== 1 && (
                <Flex justify="center">
                  <Typography.Text
                    type="secondary"
                    style={{ color: "#27ae60" }}
                  >
                    {t("paymentPlan.changeToAnnualPayment")}
                    <b>
                      {t("paymentPlan.saveAmount", {
                        amount:
                          requestData.prices[0].unit_amount * 12 -
                          requestData.prices[1].unit_amount,
                      })}
                    </b>
                  </Typography.Text>
                </Flex>
              )}
          </Col>
        </Row>
      </Card>
    </>
  );
}

/*
<ChoosenProduct
        t={t}
        paymentPlan={appContext.paymentPlan}
        extra={
          <Button
            type="link"
            icon={<CreditCardOutlined />}
            loading={isRequestingBillingDetails}
            onClick={() => {
              setIsRequestingBillingDetails(true);

              myFetch({
                method: "GET",
                url: "/payment/portal",
              })
                .then((response) => (window.location.href = response.url))
                .catch(() => {
                  setIsRequestingBillingDetails(false);
                  showUnkownErrorNotification(notificationApi, t);
                });
            }}
          >
            {t("paymentPlan.buttonUpdateBillingDetails")}
          </Button>
        }
      />
    </>
*/
